// This file is part of Doodah Band.
//
// Doodah Band is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//
// Doodah Band is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with Doodah Band. If not, see <https://www.gnu.org/licenses/>.

// Default shader to display at startup.
// This also includes the vertex shader, which is not normally seen by users.

// Simple pass-through vertex shader to feed values to user's fragment shader.
const char VERTEX_SHADER[] =
"#version 100\n"
"\n"
"attribute vec2 xy_vertex;\n"
"\n"
"varying vec2 xy;"
"\n"
"void main() {\n"
"	gl_Position = vec4(xy_vertex.x*2.0-1.0, xy_vertex.y*2.0-1.0, 0.0, 1.0);\n"
"	xy = xy_vertex;\n"
"}\n";

// Fallback shader used when user has not provided one.
// Things it would be nice for this to provide:
// - all available uniforms, samplers, varyings
//   (for convenience and built-in documentation)
// - a pretty demo shader that looks fun on first startup
// - a moderately well explained example for novices
// - a good starting point for experienced users
//   (provide common boilerplate to save tedium)
const char FRAGMENT_SHADER[] =
"#version 100\n"
"precision mediump float;\n"
"// Note that this is OpenGL ES 2.0 GLSL!\n"
"\n"
"// Time in seconds since startup.\n"
"uniform float time;\n"
"// Average loudness of music, zero to one.\n"
"uniform float amplitude;\n"
"// Resolution of display in pixels.\n"
"uniform vec2 resolution;\n"
"\n"
"// Texture containing loudness of sound frequencies in music.\n"
"// Low pitches on the left, high on the right, values zero to one.\n"
"// Texture is one pixel high.\n"
"uniform sampler2D fft;\n"
"// Slightly averaged version of `fft` to make it less flickery.\n"
"uniform sampler2D smooth_fft;\n"
"// Square RGB white noise texture.\n"
"uniform sampler2D noise;\n"
"// Texture loaded from user provided image.\n"
"uniform sampler2D image;\n"
"// Render from last frame.\n"
"uniform sampler2D frame;\n"
"\n"
"// UV coordinates, origin at bottom left.\n"
"varying vec2 xy;\n"
"\n"
"void main() {\n"
"	// Stretch input coordinates to compensate for non-square windows.\n"
"	// Resulting coordinates put origin at centre of window.\n"
"	// y runs from -0.5 to 0.5, while x's range will vary with window shape.\n"
"	vec2 aspect_correct = vec2((xy.x-0.5)*resolution.x/resolution.y, xy.y-0.5);\n"
"\n"
"	// Mirror coordinates around x and y axes to hide some artefacts later on.\n"
"	vec2 mirrored = abs(aspect_correct);\n"
"\n"
"	// Convert to something like polar coordinates.\n"
"	float angle = atan(mirrored.x/mirrored.y)/3.14159;\n"
"	float depth = 0.2/length(mirrored) + time*0.5;\n"
"\n"
"	// Amount of centre glowiness.\n"
"	// A constant background of this stops everything fading to grey.\n"
"	// Some extra is added on top to make the centre pulse to music.\n"
"	float glow = (1.0-length(mirrored))*0.5 +\n"
"		min(texture2D(fft, vec2(angle*2.0, 0.0)).r*40.0, 1.0)*0.5 +\n"
"		min(texture2D(smooth_fft, vec2(angle*2.0, 0.0)).r*40.0, 1.0)*0.5;\n"
"\n"
"	// Calculate final output colour.\n"
"	// We map noise onto the polar coordinates to get a tube.\n"
"	// Two layer use different scale factors to get a parallax effect.\n"
"	// Finally, we add the white glow on top.\n"
"	gl_FragColor = texture2D(noise, vec2(angle + time*0.04, depth))*0.5 +\n"
"		texture2D(noise, vec2(angle - time*0.02, depth)/8.0)*0.5*(0.8+amplitude*0.2) +\n"
"		vec4(glow);\n"
"}\n";
